.TH DW 1L "04 May 2019"
.\"======================================================================
.SH "NAME"
dw \- find duplicate words
.\"======================================================================
.SH "SYNOPSIS"
.B dw
[
.B \-\-help
]
[
.B \-\-keep-lettercase
]
[
.B \-\-list-words
]
.if n .ti +3n
[
.B \-\-skip-punctuation
]
[
.B \-\-version
]
.if n .ti +3n
.if t .ti +.22i
[
.BI \-\-word-pattern " '[initial][rest]'"
]
.if n .ti +3n
[
.BI \-\-word-pattern " pattern-file"
]
.if n .ti +4n
.if t .ti +.5i
.I "< infile > outfile"
.if n .ti +2n
.if t .ti +.22i
or
.if n .ti +4n
.if t .ti +.5i
.I "infile1 infile2 infile3 .\|.\|.\| > outfile"
.\"=====================================================================
.SH DESCRIPTION
.PP
.B dw
is a handy tool for finding a common typographical
error in documentation.  If filters its standard
input, or a list of files, printing on standard
output only those words that appear two or more
times in succession.  Each word is prefixed by its
filename and line number(s).  For
.IR stdin ,
the filename is a dash.
.PP
A
.I word
starts with a letter or underscore, and is
followed by zero or more letters, underscores, or
digits.  Letter case is ignored.  Program options
can modify that behavior.
.\"======================================================================
.SH "OPTIONS"
Command-line options may be abbreviated to any
unique prefix, with one or two leading dashes, and
their lettercase is
.I not
significant.  Thus,
.BR \-v ,
.BR \-V ,
and
.BR \-VeRs
are equivalent to
.BR \-\-version .
.\"-----------------------------------------------
.TP \w'\-\-skip-punctuation'u+3n
.BR \-\-help " or " \-\-?
Display a help message on
.I stdout
and exit.
.\"-----------------------------------------------
.TP
.B \-\-keep-lettercase
Preserve input lettercase, overriding the normal
behavior of comparing words in a single
lettercase.
.\"-----------------------------------------------
.TP
.B \-\-list-words
Print each recognized word on a separate line
on
.IR stdout .
Use that option to check the word recognition
algorithm, as well as to extract text from files
that contain punctuation and other material.  The
word list can often usefully be passed to other
text-processing tools, such as
.BR aspell (1),
.BR grep (1),
.BR hunspell (1),
.BR ispell (1),
.BR sort (1),
.BR spell (1),
and
.BR uniq (1).
.\"-----------------------------------------------
.TP
.B \-\-skip-punctuation
Collapse to a single space all sequences of
characters that are neither whitespace nor word
characters.  Without that option, the input
.I "\fCsally(sally)sally/sally\fP"
does not produce a doubled word report; with it,
.I three
such instances are diagnosed.
.\"-----------------------------------------------
.TP
.B \-\-version
Print the program name, version, and copyright
information on
.I stdout
and exit.
.\"-----------------------------------------------
.TP
.BI \-\-word-pattern " \fC'[initial][rest]'\fP" "\fR or \fP" \fB\-\-word-pattern\fP " \fIfile\fP"
Specify an alternate pattern for recognizing the
initial and following characters in words.  The
pattern is given in the following argument as two
bracketed character sets, in which dashes
represent ranges of consecutive characters.
Alternatively, the pattern may be read from a
user-supplied file, or from a file in the
.B dw
installation tree.  In the latter case, it is
found relative to the location of the executable
program, in a directory named something like
\&\fC../share/lib/dw/dw-2.00\fP.
.IP
Pattern files may contain leading comment lines
beginning with a sharp character (\fC#\fP):
reading terminates after the first noncomment
line.
.IP
For installed pattern files, a suffix \fC.pat\fP
is automatically supplied, and the file contents
are in Unicode UTF-8 encoding.  Such files are
named by the
.I lowercase
English name of the language, or by its ISO 639-1
alpha-2 code.  Those codes are often identical to
the ISO 3166-1 alpha-2 codes that are used for
Internet top-level domain names.
Thus, \&\fB\-\-word-pattern\fP
\&\fCspanish\fP and \&\fB\-\-word-pattern\fP
\&\fCes\fP find equivalent pattern files,
\&\fCspanish.pat\fP and \&\fCes.pat\fP, for that
language.
.IP
The default pattern, similar to that for
identifiers in many programming languages, and
English-language text, is
.IR \fC[A-Za-z_][A-Za-z_0-9]\fP .
That means an initial letter or underscore,
followed by any number of letters, underscores, or
digits.  Whitespace around the bracketed patterns
is ignored.
.IP
Any character in the pattern,
.I except
the character range dash, may be represented by an
.I "octal escape sequence"
of the form
.IR "\fC\eooo\fP" ,
.IR "\fC\eoo\fP" ,
or
.IR "\fC\eo\fP" ,
where
.I o
is an octal digit, or by a
.I "hexadecimal escape sequence"
of the form
.IR "\exhh" ,
where
.I h
is a hexadecimal digit in
.I \fC0-9a-f\fP
(ignoring lettercase), or by a
.I "literal escape sequence"
of the form
.IR "\fC\ec\fP"
where
.I c
is any single character, representing itself.  The
escape character checks are made in that order, so
\&\fC\ex\fP begins a hexadecimal sequence, rather
than being a literal \fCx\fP.
.IP
Thus, the vowels
.B aeiou
may be represented by normal characters
\&\fCaeiou\fP, by the literal sequence
.IR \fC\ea\ee\ei\eo\eu\fP ,
by the octal sequence
.IR \fC\e141\e145\e151\e157\e165\fP ,
by the hexadecimal sequence
.IR \fC\ex61\ex65\ex69\ex6f\ex75\fP ,
or any mixture thereof.
.\"======================================================================
.SH "BUGS AND FEATURES"
.B dw
assumes one-byte characters, as used in ASCII and
the various ISO Latin-\fIn\fP character sets.
Thus, it normally does not recognize words that
contain multibyte characters, such as accented
letters in Unicode UTF-8 encoding.  Neverthless,
if you put multibyte characters into the
.B \-\-word-pattern
value, they will be treated as word constituents,
but so will any other characters that have the
same leading bytes.  While not strictly correct,
for many languages, most words are correctly
identified, because the UTF-8 encodings group
accented letters together.
.PP
Here are examples of additional characters needed
in word patterns (some glyphs may be lost due
to output device and/or font limitations). All of
them are recognized by
.BR dw ,
with language names in lowercase:
.TP \w'Swedish_(se_or_sv)'u+3n
.B "Arabic (ar)"
.I "numerous letters and letter variants"
.TP
.B "Danish (da)"
.\" FAILS: .AM
.\" FAILS: \fC e\*[']ec\*[,]c\fP
.\" FAILS: \fCÉÆØÅéæøå\fP
.\" OKAY: \fC\[u0045_0301]\[u00C6]\[u00D8]\[u0041_030A]\[u0065_0301]\[u00E6]\[u00F8]\[u0061_030A]\fP
\&\fC\['E]\[AE]\[/O]\[oA] \['e]\[ae]\[/o]\[oa]\fP
.TP
.B default
\&\fC[A-Za-z_][A-Za-z_0-9]\fP
.TP
.B "Dutch (nl)"
.I "none normally needed, but sometimes, ligatures \fC\[IJ] \[ij]\fP"
.TP
.B "English (en)"
.I "none normally needed, but rarely, \fC\[AE]\[:I]\[OE] \[ae]\[:i]\[oe]\fP"
.TP
.B "Finnish (fi)"
\&\fC\['E]\[oA]\[:A]\[:O]\[vS]\[vZ] \['e]\[oa]\[:a]\[:o]\[vs]\[vz]\fP
.TP
.B "French (fr)"
.\" FAILS: \fCÂÉÈÊÇÔ âéèêçô\fP
\&\fC\[^A]\[:A]\['E]\[`E]\[^E]\[:E]\[,C]\[^I]\[:I]\[^O]\[:O]\[^U]\[:U]\[:Y] \[AE]\[OE]
\[^a]\[:a]\['e]\[`e]\[^e]\[:e]\[,c]\[^i]\[:i]\[^o]\[:o]\[^u]\[:u]\[:y] \[ae]\[oe]\fP
.TP
.B "German (de)"
.\" FAILS: \fCÄÖÜ äöüß\fP
\&\fC\[:A]\[:O]\[:U] \[:a]\[:o]\[:u] \[ss]\fP
.TP
.B "Greek (el)"
\&\fC[24 Greek letters, in each case]\fP
.TP
.B "Hebrew (he)"
.I "22 letters in one case, with 5 word-ending variants, plus apostrophe"
.TP
.B "Hungarian (hu)"
\&\fC\['A]\['E]\['I]\['O]\['U]\[:O]\[:U] \['a]\['e]\['i]\['o]\['u]\[:o]\[:u],\fP
.I "plus long double acute accent on O U o u"
.TP
.B "Icelandic (is)"
.\" FAILS: \fCÁÐÉÓÚÝÞÖ áðéóúýþö\fP
\&\fC\['A]\[-D]\['E]\['O]\['U]\['Y]\[TP]\[:O] \['a]\[Sd]\['e]\['o]\['u]\['y]\[Tp]\[:o]\fP
.TP
.B "Irish (ga)"
\&\fC\['A]\['E]\['I]\['O]\['U] \['a]\['e]\['i]\['o]\['u]\fP
.TP
.B "Italian (it)"
\&\fC\['E]\[`E]\['I]\[`I]\[^I]\['O]\[`O]\['U]\[`U] \['e]\[`e]\['i]\[`i]\[^i]\['o]\[`o]\['u]\[`u]\fP
.TP
.B "Latin-1"
.I "all accented letters from ISO-8859-1 (Latin-1) encoding"
.TP
.B "Lisp"
\&\fC-%$:\fP
.TP
.B "Maltese (mt)"
.I "four accented letters in two cases, plus apostrophe"
.TP
.B "Norwegian (no)"
\&\fC\[^A]\['E]\[`E]\[^E]\['O]\[`O]\[^O]\[AE]\[/O]\[oA] \[^a]\['e]\[`e]\[^e]\['o]\[`o]\[^o]\[ae]\[/o]\[oa]\fP
.TP
.B "Polish (pl)"
.I "acute accent on C N O S Z c n o s z, ogonek on A E a e, overdot on Z z, and stroke on L l"
.TP
.B "Portuguese (pt)"
\&\fC\['A]\[^A]\[~A]\[,C]\['E]\[^E]\['I]\['O]\[^O]\[~O]\['U] \['a]\[^a]\[~a]\[,c]\['e]\[^e]\['i]\['o]\[^o]\[~o]\['u]\fP
.TP
.B "Russian (ru)"
.I "Cyrillic alphabet"
.TP
.B "Scots (sc)"
.I "none needed"
.TP
.B "Spanish (es)"
.\" FAILS: \fCÁÉÍÓÚÑ áéíóúñ\fP
\&\fC\['A]\['E]\['I]\['O]\['U]\[~N] \['a]\['e]\['i]\['o]\['u]\[~n]\fP
.TP
.B "Swahili (sw)"
\&\fC'?\fP
.TP
.B "Swedish (se \fRor\fP sv)"
\&\fC\['E]\[oA]\[:A]\[:O] \['e]\[oa]\[:a]\[:o]\fP
.TP
.B "Welsh (cy)"
\&\fC' \['A]\[`A]\[^A]\[:A]\['E]\[`E]\[^E]\[:E]\['I]\[`I]\[^I]\[:I]\['O]\[`O]\[^O]\[:O]\['U]\[`U]\[^U]\[:U]
\['a]\[`a]\[^a]\[:a]\['e]\[`e]\[^e]\[:e]\['i]\[`i]\[^i]\[:i]\['o]\[`o]\[^o]\[:o]\['u]\[`u]\[^u]\[:u]\fP
.PP
For languages that use characters outside the
Latin-alphabet family, the patterns include the
.B dw
default pattern, because mixed-with-English texts
are common.
.PP
The languages above include two that are written
from right to left (Arabic and Hebrew, each with
distinctive non-Latin scripts).
.\"======================================================================
.SH "AUTHOR"
.nf
Nelson H. F. Beebe
University of Utah
Department of Mathematics, 110 LCB
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
USA
WWW URL: \fChttp://www.math.utah.edu/~beebe\fP
Tel: +1 801 581 5254
FAX: +1 801 581 4148
Email: \fCbeebe@math.utah.edu, beebe@acm.org, beebe@computer.org\fP
.fi
.\"==============================[The End]==============================
